import PropTypes from 'prop-types';
import createMapper from 'system-classnames';
import React from 'react';
import classnames from 'classnames';
import Octicon, { GitMerge, IssueClosed, IssueOpened, IssueReopened } from '@github/octicons-react';
import { arc, pie } from 'd3-shape';

var colors = {
  bodytext: '#24292e',
  black: '#1b1f23',
  white: '#fff',
  gray: ['#fafbfc', '#f6f8fa', '#e1e4e8', '#d1d5da', '#959da5', '#6a737d', '#586069', '#444d56', '#2f363d', '#24292e'],
  blue: ['#f1f8ff', '#dbedff', '#c8e1ff', '#79b8ff', '#2188ff', '#0366d6', '#005cc5', '#044289', '#032f62', '#05264c'],
  green: ['#f0fff4', '#dcffe4', '#bef5cb', '#85e89d', '#34d058', '#28a745', '#22863a', '#176f2c', '#165c26', '#144620'],
  yellow: ['#fffdef', '#fffbdd', '#fff5b1', '#ffea7f', '#ffdf5d', '#ffd33d', '#f9c513', '#dbab09', '#b08800', '#735c0f'],
  orange: ['#fff8f2', '#ffebda', '#ffd1ac', '#ffab70', '#fb8532', '#f66a0a', '#e36209', '#d15704', '#c24e00', '#a04100'],
  red: ['#ffeef0', '#ffdce0', '#fdaeb7', '#f97583', '#ea4a5a', '#d73a49', '#cb2431', '#b31d28', '#9e1c23', '#86181d'],
  purple: ['#f5f0ff', '#e6dcfd', '#d1bcf9', '#b392f0', '#8a63d2', '#6f42c1', '#5a32a3', '#4c2889', '#3a1d6e', '#29134e'],
  blackfade15: 'rgba(27, 31 ,35, 0.15)',
  blackfade20: 'rgba(27, 31 ,35, 0.20)',
  whitefade15: 'rgba(255, 255 ,255, 0.15)'
};

var blue = colors.blue,
    gray = colors.gray,
    green = colors.green,
    purple = colors.purple,
    red = colors.red,
    yellow = colors.yellow;


colors.border = {
  'black-fade': colors.blackfade15,
  blue: blue[5],
  'blue-light': blue[2],
  green: green[4],
  'green-light': green[3], // FIXME desaturate($green-300, 40%)
  red: red[5],
  'red-light': red[3], // FIXME desaturate($red-300, 60%)
  purple: purple[5],
  yellow: yellow[3], // FIXME desaturate($yellow-300, 60%)
  gray: gray[2],
  'gray-light': gray[2], // FIXME lighten($gray-200, 3%)
  'gray-dark': gray[3],
  'gray-darker': gray[7]
};

colors.bg = {
  white: colors.white,
  'gray-light': gray[0],
  gray: gray[1],
  'gray-dark': gray[9],
  purple: purple[5],
  blue: blue[5],
  green: green[5],
  yellow: yellow[5],
  // orange: orange[7],
  red: red[5],
  'purple-light': purple[0],
  'blue-light': blue[0],
  'green-light': green[1],
  'yellow-light': yellow[2],
  'red-light': red[1]
};

var theme = {
  breakpoints: ['544px', '768px', '1012px', '1280px'],
  maxWidths: {
    small: '544px',
    medium: '768px',
    large: '1012px',
    xlarge: '1280px'
  },
  fonts: ['-apple-system', 'BlinkMacSystemFont', 'Segoe UI', 'Helvetica', 'Arial', 'sans-serif', 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol'],
  lineHeight: 1.5,
  colors: colors,
  radii: [0, 3, 6],
  space: [0, 4, 8, 16, 24, 32, 40, 48],
  fontSizes: [12, 14, 16, 20, 24, 32, 40, 48]
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var breakpoints = [null, 'sm', 'md', 'lg', 'xl'];

var oneOrMoreOf = function oneOrMoreOf(type) {
  return PropTypes.oneOfType([type, PropTypes.arrayOf(type)]);
};

var oneOrMoreNumbers = oneOrMoreOf(PropTypes.number);

var createMapperWithPropTypes = function createMapperWithPropTypes(props) {
  var mapper = createMapper({
    breakpoints: breakpoints,
    props: props,
    getter: function getter(_ref) {
      var breakpoint = _ref.breakpoint,
          prop = _ref.prop,
          value = _ref.value;
      return breakpoint ? [prop, breakpoint, value].join('-') : [prop, value].join('-');
    }
  });
  mapper.propTypes = props.reduce(function (propTypes, prop) {
    propTypes[prop] = oneOrMoreNumbers;
    return propTypes;
  }, {});
  return mapper;
};

var marginProps = ['m', 'mt', 'mr', 'mb', 'ml', 'mx', 'my'];
var paddingProps = ['p', 'pt', 'pr', 'pb', 'pl', 'px', 'py'];

var mapWhitespaceProps = createMapperWithPropTypes(marginProps.concat(paddingProps));

function stylizer(propsToPass) {
  return function (props) {
    var copy = _extends({}, props);
    copy.style = propsToPass.reduce(function (acc, prop) {
      if (prop in props) {
        acc[prop] = props[prop];
        delete copy[prop];
      }
      return acc;
    }, props.style || {});
    return copy;
  };
}

var borderColors = Object.keys(colors.border);

var styleProps = ['width', 'minWidth', 'maxWidth', 'height', 'minHeight', 'maxHeight'];
var stylize = stylizer(styleProps);

function unique(values) {
  return values.filter(function (v, i) {
    return values.indexOf(v) === i;
  });
}

function getBorderClass(value, colorValue) {
  if (Array.isArray(value)) {
    return unique(value.map(getBorderClass));
  } else if (typeof value === 'boolean') {
    return value ? 'border' : 'border-0';
  } else if (value) {
    return 'border-' + value;
  } else if (colorValue) {
    return 'border';
  }
}

var Block = function Block(props) {
  var _mapWhitespaceProps = mapWhitespaceProps(props),
      _mapWhitespaceProps$t = _mapWhitespaceProps.tag,
      Tag = _mapWhitespaceProps$t === undefined ? 'div' : _mapWhitespaceProps$t,
      children = _mapWhitespaceProps.children,
      className = _mapWhitespaceProps.className,
      bg = _mapWhitespaceProps.bg,
      border = _mapWhitespaceProps.border,
      borderColor = _mapWhitespaceProps.borderColor,
      borderRadius = _mapWhitespaceProps.borderRadius,
      display = _mapWhitespaceProps.display,
      fg = _mapWhitespaceProps.fg,
      position = _mapWhitespaceProps.position,
      shadow = _mapWhitespaceProps.shadow,
      rest = objectWithoutProperties(_mapWhitespaceProps, ['tag', 'children', 'className', 'bg', 'border', 'borderColor', 'borderRadius', 'display', 'fg', 'position', 'shadow']);

  var _stylize = stylize(rest),
      style = _stylize.style;

  return React.createElement(
    Tag,
    {
      className: classnames(className, getBorderClass(border, borderColor), borderColor && 'border-' + borderColor, display && 'd-' + display, bg && 'bg-' + bg, fg && 'text-' + fg, typeof borderRadius === 'number' && 'rounded-' + borderRadius, position && 'position-' + position, shadow && (shadow === 'small' ? 'box-shadow' : 'box-shadow-' + shadow)),
      style: style
    },
    children
  );
};

Block.propTypes = _extends({
  bg: PropTypes.string,
  border: PropTypes.oneOfType([PropTypes.bool, oneOrMoreOf(PropTypes.oneOf(['top', 'right', 'bottom', 'left']))]),
  borderColor: PropTypes.oneOf(borderColors),
  borderRadius: PropTypes.oneOf([0, 1, 2]),
  children: PropTypes.node,
  display: PropTypes.oneOf(['inline', 'inline-block', 'none']),
  fg: PropTypes.string,
  position: PropTypes.oneOf(['absolute', 'fixed', 'relative']),
  shadow: PropTypes.oneOf(['small', 'medium', 'large', 'extra-large'])
}, mapWhitespaceProps.propTypes);

var _iteratorNormalCompletion = true;
var _didIteratorError = false;
var _iteratorError = undefined;

try {
  for (var _iterator = styleProps[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
    var prop = _step.value;

    Block.propTypes[prop] = PropTypes.number;
  }
} catch (err) {
  _didIteratorError = true;
  _iteratorError = err;
} finally {
  try {
    if (!_iteratorNormalCompletion && _iterator.return) {
      _iterator.return();
    }
  } finally {
    if (_didIteratorError) {
      throw _iteratorError;
    }
  }
}

var Box = function Box(props) {
  return React.createElement(Block, props);
};

Box.defaultProps = {
  bg: 'white',
  border: true,
  borderRadius: 1
};

Box.propTypes = Block.propTypes;

var Avatar = function Avatar(props) {
  var alt = props.alt,
      isChild = props.isChild,
      _props$size = props.size,
      size = _props$size === undefined ? 20 : _props$size,
      src = props.src;


  var className = classnames('avatar', {
    'avatar-small': size <= 24,
    'avatar-child': isChild
  });

  return React.createElement('img', { className: className, alt: alt, src: src, width: size, height: size });
};

Avatar.propTypes = {
  alt: PropTypes.string,
  isChild: PropTypes.bool,
  size: PropTypes.number,
  src: PropTypes.string
};

function Button(_ref) {
  var _ref$tag = _ref.tag,
      Tag = _ref$tag === undefined ? 'button' : _ref$tag,
      children = _ref.children,
      size = _ref.size,
      block = _ref.block,
      linkStyle = _ref.linkStyle,
      grouped = _ref.grouped,
      scheme = _ref.scheme,
      onClick = _ref.onClick,
      disabled = _ref.disabled,
      props = objectWithoutProperties(_ref, ['tag', 'children', 'size', 'block', 'linkStyle', 'grouped', 'scheme', 'onClick', 'disabled']);

  var classes = classnames({
    btn: !linkStyle,
    'btn-link': linkStyle,
    'btn-sm': size === 'sm',
    'btn-large': size === 'large',
    'btn-block': block,
    'BtnGroup-item': grouped
  }, scheme ? 'btn-' + scheme : null);

  return React.createElement(
    Tag,
    _extends({}, props, { type: 'button', disabled: disabled, onClick: disabled ? undefined : onClick, className: classes }),
    children
  );
}

Button.propTypes = {
  block: PropTypes.bool,
  grouped: PropTypes.bool,
  scheme: PropTypes.string,
  size: PropTypes.oneOf(['sm', 'large']),
  tag: PropTypes.oneOf(['button', 'a', 'summary']),
  linkStyle: PropTypes.bool,
  disabled: PropTypes.bool,
  onClick: PropTypes.func
};

var ButtonDanger = function ButtonDanger(props) {
  return React.createElement(Button, _extends({}, props, { scheme: 'danger' }));
};

var ButtonPrimary = function ButtonPrimary(props) {
  return React.createElement(Button, _extends({}, props, { scheme: 'primary' }));
};

var ButtonOutline = function ButtonOutline(props) {
  return React.createElement(Button, _extends({}, props, { scheme: 'outline' }));
};

var ButtonLink = function ButtonLink(props) {
  return React.createElement(Button, _extends({}, props, { linkStyle: true }));
};

var oppositeEdge = {
  top: 'Bottom',
  right: 'Left',
  bottom: 'Top',
  left: 'Right'
};

var perpendicularEdge = {
  top: 'Left',
  right: 'Top',
  bottom: 'Left',
  left: 'Top'
};

var offsetSpacing = theme.space[2];

function getEdgeAlign(location) {
  var _location$split = location.split('-'),
      _location$split2 = slicedToArray(_location$split, 2),
      edge = _location$split2[0],
      align = _location$split2[1];

  return [edge, align];
}

function getPosition(edge, align) {
  var _ref;

  var opposite = oppositeEdge[edge].toLowerCase();
  var perp = perpendicularEdge[edge].toLowerCase();
  var offsetProp = align || perp;
  var offsetValue = align ? offsetSpacing : '50%';
  return _ref = {}, defineProperty(_ref, opposite, '100%'), defineProperty(_ref, offsetProp, offsetValue), _ref;
}

function Caret(props) {
  var borderColor = props.borderColor,
      borderWidth = props.borderWidth,
      fill = props.fill,
      location = props.location,
      size = props.size;

  var _getEdgeAlign = getEdgeAlign(location),
      _getEdgeAlign2 = slicedToArray(_getEdgeAlign, 2),
      edge = _getEdgeAlign2[0],
      align = _getEdgeAlign2[1];

  var perp = perpendicularEdge[edge];

  var style = _extends({
    pointerEvents: 'none',
    position: 'absolute'
  }, getPosition(edge, align), defineProperty({}, 'margin' + perp, align ? null : -size));

  // note: these arrays represent points in the form [x, y]
  var a = [-size, 0];
  var b = [0, size];
  var c = [size, 0];

  // spaces are optional in path `d` attribute, and points are
  // represented in the form `x,y` -- which is what the arrays above
  // become when stringified!
  var triangle = 'M' + a + 'L' + b + 'L' + c + 'L' + a + 'Z';
  var line = 'M' + a + 'L' + b + 'L' + c;

  var transform = {
    top: 'translate(' + [size, size * 2] + ') rotate(180)',
    right: 'translate(' + [0, size] + ') rotate(-90)',
    bottom: 'translate(' + [size, 0] + ')',
    left: 'translate(' + [size * 2, size] + ') rotate(90)'
  }[edge];

  return React.createElement(
    'svg',
    { width: size * 2, height: size * 2, style: style },
    React.createElement(
      'g',
      { transform: transform },
      React.createElement('path', { d: triangle, fill: fill }),
      React.createElement('path', { d: line, fill: 'none', stroke: borderColor, strokeWidth: borderWidth })
    )
  );
}

Caret.locations = ['top', 'top-left', 'top-right', 'right', 'right-top', 'right-bottom', 'bottom', 'bottom-left', 'bottom-right', 'left', 'left-top', 'left-bottom'];

Caret.defaultProps = {
  borderColor: colors.gray[2],
  borderWidth: 1,
  fill: colors.white,
  location: 'bottom',
  size: theme.space[2]
};

Caret.propTypes = {
  borderColor: PropTypes.string,
  borderWidth: PropTypes.number,
  location: PropTypes.oneOf(Caret.locations),
  fill: PropTypes.string,
  size: PropTypes.number
};

var borderValuesByName = _extends({}, colors.border, {
  true: colors.gray[2] // default
});

var bgValuesByName = colors.bg;

function CaretBox(props) {
  var bg = props.bg,
      border = props.border,
      location = props.caret,
      children = props.children,
      boxProps = objectWithoutProperties(props, ['bg', 'border', 'caret', 'children']);


  var borderColorName = Array.isArray(border) ? border.filter(function (value) {
    return value in borderValuesByName;
  }).pop() : border;

  var caretProps = {
    location: location,
    borderColor: borderValuesByName[borderColorName],
    fill: bgValuesByName[bg]
  };

  return React.createElement(
    Box,
    _extends({}, boxProps, { bg: bg, border: border }),
    children,
    React.createElement(Caret, caretProps)
  );
}

CaretBox.propTypes = _extends({}, Box.propTypes, {
  caret: Caret.propTypes.location
});

CaretBox.defaultProps = _extends({}, Box.defaultProps, {
  position: 'relative'
});

var CircleOcticon = function CircleOcticon(_ref) {
  var size = _ref.size,
      bg = _ref.bg,
      color = _ref.color,
      rest = objectWithoutProperties(_ref, ['size', 'bg', 'color']);

  var className = classnames('circle d-flex flex-items-center flex-justify-center', bg ? 'bg-' + bg : null, color ? 'text-' + color : null);
  return React.createElement(
    'div',
    { style: { width: size + 'px', height: size + 'px' }, className: className },
    React.createElement(Octicon, _extends({ size: size }, rest))
  );
};

CircleOcticon.propTypes = _extends({}, Octicon.propTypes, {
  // FIXME: we should be able to infer the size from the
  // rendered Octicon so that we can support
  // size={'small|medium|large'}
  size: PropTypes.number
});

function getRenderer(children) {
  return typeof children === 'function' ? children : function () {
    return children;
  };
}

var Details = function (_React$Component) {
  inherits(Details, _React$Component);

  function Details(props) {
    classCallCheck(this, Details);

    var _this = possibleConstructorReturn(this, (Details.__proto__ || Object.getPrototypeOf(Details)).call(this, props));

    _this.state = { open: Boolean(props.open) };
    _this.toggle = _this.toggle.bind(_this);
    return _this;
  }

  createClass(Details, [{
    key: 'toggle',
    value: function toggle(event) {
      if (event) {
        event.preventDefault();
      }
      this.setState({ open: !this.state.open });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          className = _props.className,
          children = _props.children,
          _props$render = _props.render,
          render = _props$render === undefined ? getRenderer(children) : _props$render,
          props = objectWithoutProperties(_props, ['className', 'children', 'render']);
      var open = this.state.open;


      return React.createElement(
        'details',
        _extends({}, props, { className: classnames('details-reset', className), open: open }),
        render({ open: open, toggle: this.toggle })
      );
    }
  }]);
  return Details;
}(React.Component);


Details.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  className: PropTypes.string,
  open: PropTypes.bool,
  render: PropTypes.func
};

var arrowStyles = {
  content: '',
  border: '4px solid',
  borderRightColor: 'transparent',
  borderLeftColor: 'transparent',
  borderBottomColor: 'transparent',
  width: '0',
  height: '0'
};

var Dropdown = function Dropdown(_ref) {
  var title = _ref.title,
      scheme = _ref.scheme,
      children = _ref.children;
  return React.createElement(
    'div',
    { className: 'BtnGroup' },
    React.createElement(
      Details,
      { className: 'details-reset BtnGroup-form d-flex' },
      function (_ref2) {
        var toggle = _ref2.toggle;
        return React.createElement(
          React.Fragment,
          null,
          React.createElement(
            Button,
            { tag: 'summary', scheme: scheme, grouped: true, onClick: toggle },
            title,
            ' ',
            React.createElement('div', { className: 'd-inline-block v-align-middle', style: arrowStyles })
          ),
          React.createElement(
            'div',
            {
              className: 'border box-shadow position-absolute px-3 py-2 bg-white mt-1 rounded-1 list-style-none',
              style: { zIndex: 99999 }
            },
            React.createElement(
              'div',
              { className: 'position-absolute ml-0 mt-n2' },
              React.createElement(Caret, { edge: 'top', align: 'start' })
            ),
            children
          )
        );
      }
    )
  );
};

var defaultFill = colors.gray[4];

var fillForState = {
  error: colors.red[5],
  queued: colors.yellow[7],
  pending: colors.yellow[7],
  failure: colors.red[5],
  success: colors.green[5],
  unknown: defaultFill
};

var DonutSlice = function DonutSlice(props) {
  var children = props.children,
      d = props.d,
      state = props.state,
      _props$fill = props.fill,
      fill = _props$fill === undefined ? fillForState[state] || defaultFill : _props$fill,
      value = props.value;

  return React.createElement(
    'path',
    { d: d, fill: fill, 'data-value': value },
    children
  );
};

DonutSlice.states = Object.keys(fillForState);

DonutSlice.propTypes = {
  // <title> is really the only thing that should be acceptable here
  children: PropTypes.shape({ type: 'title' }),
  d: PropTypes.string,
  fill: PropTypes.string,
  state: PropTypes.oneOf(DonutSlice.states),
  value: PropTypes.number
};

function mapData(data) {
  return Object.keys(data).map(function (key) {
    return React.createElement(DonutSlice, { key: key, state: key, value: data[key] });
  });
}

var DonutChart = function DonutChart(props) {
  var data = props.data,
      _props$children = props.children,
      children = _props$children === undefined ? mapData(data) : _props$children,
      _props$size = props.size,
      size = _props$size === undefined ? 30 : _props$size;


  var radius = size / 2;
  var innerRadius = radius - 6;

  var pie$$1 = pie().value(function (child) {
    return child.props.value;
  });

  // coerce the children into an array
  var childList = React.Children.toArray(children);
  var arcData = pie$$1(childList);
  var arc$$1 = arc().innerRadius(innerRadius).outerRadius(radius);

  var slices = childList.map(function (child, i) {
    return React.cloneElement(child, { d: arc$$1(arcData[i]) });
  });

  return React.createElement(
    'svg',
    { width: size, height: size },
    React.createElement(
      'g',
      { transform: 'translate(' + radius + ',' + radius + ')' },
      slices
    )
  );
};

DonutChart.propTypes = {
  // require elements, not mixed content: <DonutSlice>, <title>, etc.
  children: oneOrMoreOf(PropTypes.element),
  data: PropTypes.objectOf(PropTypes.number),
  size: PropTypes.number
};

var FlexContainer = function FlexContainer(_ref) {
  var children = _ref.children,
      inline = _ref.inline,
      wrap = _ref.wrap,
      direction = _ref.direction,
      justifyContent = _ref.justifyContent,
      alignItems = _ref.alignItems,
      alignContent = _ref.alignContent,
      rest = objectWithoutProperties(_ref, ['children', 'inline', 'wrap', 'direction', 'justifyContent', 'alignItems', 'alignContent']);

  var classes = classnames({
    'd-flex': !inline,
    'd-inline-flex': inline
  }, wrap && 'flex-' + wrap, direction && 'flex-' + direction, justifyContent && 'flex-justify-' + justifyContent, alignItems && 'flex-items-' + alignItems, alignContent && 'flex-content-' + alignContent);

  return React.createElement(
    Block,
    _extends({}, rest, { className: classes }),
    children
  );
};

FlexContainer.propTypes = {
  inline: PropTypes.bool,
  wrap: PropTypes.oneOf(['wrap', 'nowrap']),
  direction: PropTypes.oneOf(['row', 'row-reverse', 'column']),
  justifyContent: PropTypes.oneOf(['start', 'end', 'center', 'between', 'around']),
  alignItems: PropTypes.oneOf(['start', 'end', 'center', 'baseline', 'stretch']),
  alignContent: PropTypes.oneOf(['start', 'end', 'center', 'between', 'around', 'stretch'])
};

var FlexItem = function FlexItem(_ref) {
  var _ref$tag = _ref.tag,
      Tag = _ref$tag === undefined ? 'div' : _ref$tag,
      children = _ref.children,
      flexAuto = _ref.flexAuto,
      alignSelf = _ref.alignSelf;

  var classes = classnames({ 'flex-auto': flexAuto }, alignSelf && 'flex-self-' + alignSelf);

  return React.createElement(
    Tag,
    { className: classes },
    children
  );
};

FlexItem.propTypes = {
  flexAuto: PropTypes.bool,
  alignSelf: PropTypes.oneOf(['auto', 'start', 'end', 'center', 'baseline', 'stretch']),
  tag: PropTypes.string
};

var TextInput = function TextInput(_ref) {
  var autocomplete = _ref.autocomplete,
      autofocus = _ref.autofocus,
      block = _ref.block,
      disabled = _ref.disabled,
      id = _ref.id,
      name = _ref.name,
      placeholder = _ref.placeholder,
      required = _ref.required,
      size = _ref.size,
      value = _ref.value;
  return React.createElement('input', {
    'aria-label': placeholder,
    autoComplete: autocomplete,
    autoFocus: autofocus,
    className: classnames('form-control', {
      'input-block': block,
      'input-sm': size === 'small',
      'input-lg': size === 'large'
    }),
    disabled: disabled,
    id: id,
    name: name,
    placeholder: placeholder,
    required: required,
    type: 'text',
    value: value
  });
};

TextInput.propTypes = {
  autocomplete: PropTypes.string,
  autofocus: PropTypes.bool,
  block: PropTypes.bool,
  disabled: PropTypes.bool,
  id: PropTypes.string,
  name: PropTypes.string,
  placeholder: PropTypes.string,
  required: PropTypes.bool,
  size: PropTypes.oneOf(['small', 'large']),
  value: PropTypes.string
};

var fontSizeMap = {
  0: 6,
  1: 5,
  2: 4,
  3: 3,
  4: 2,
  5: 1,
  6: 0
};

var Text = function Text(props) {
  var _props$tag = props.tag,
      Tag = _props$tag === undefined ? 'span' : _props$tag,
      children = props.children,
      color = props.color,
      fontSize = props.fontSize,
      fontWeight = props.fontWeight,
      lineHeight = props.lineHeight,
      mono = props.mono,
      nowrap = props.nowrap,
      rest = objectWithoutProperties(props, ['tag', 'children', 'color', 'fontSize', 'fontWeight', 'lineHeight', 'mono', 'nowrap']);

  var _mapWhitespaceProps = mapWhitespaceProps(rest),
      className = _mapWhitespaceProps.className;

  var fontSizeClass = fontSize in fontSizeMap ? 'f' + fontSizeMap[fontSize] : typeof fontSize === 'number' || fontSize ? 'f' + fontSize : null;

  return React.createElement(
    Tag,
    {
      className: classnames(className, color && 'text-' + color, fontSizeClass, fontWeight && 'text-' + fontWeight, lineHeight && 'lh-' + lineHeight, mono && 'text-mono', nowrap && 'no-wrap')
    },
    children
  );
};

Text.propTypes = _extends({
  // TODO: constrain with PropTypes.oneOf()
  color: PropTypes.string,
  // TODO constrain with PropTypes.oneOf()
  fontSize: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  fontWeight: PropTypes.oneOf(['normal', 'bold']),
  lineHeight: PropTypes.oneOf(['normal', 'condensed', 'condensed-ultra']),
  mono: PropTypes.bool,
  nowrap: PropTypes.bool,
  tag: PropTypes.string
}, mapWhitespaceProps.propTypes);

var Heading = function Heading(props) {
  return React.createElement(Text, props);
};

Heading.defaultProps = {
  tag: 'h1',
  fontSize: 5,
  m: 0
};

Heading.propTypes = _extends({}, Text.propTypes);

var colorScheme = function colorScheme(scheme, outline) {
  if (outline) {
    return {
      'Label--outline-green': scheme === 'green'
    };
  } else {
    return {
      'Label--gray': scheme == null || scheme === 'gray',
      'Label--gray-darker': scheme === 'gray-darker',
      'Label--orange': scheme === 'orange',
      'bg-green': scheme === 'green'
    };
  }
};

var Label = function Label(props) {
  var outline = props.outline,
      scheme = props.scheme,
      children = props.children;

  return React.createElement(
    'span',
    { className: classnames('Label', outline ? 'Label--outline' : '', colorScheme(scheme, outline)) },
    children
  );
};

var BranchName = function BranchName(_ref) {
  var children = _ref.children,
      href = _ref.href,
      tag = _ref.tag;

  var Tag = tag === 'a' ? 'a' : 'span';

  // We don't want someone to use href on a non tag
  if (tag !== 'a') {
    href = null;
  }

  return React.createElement(
    Tag,
    { href: href, className: 'branch-name' },
    children
  );
};

var Link = function Link(_ref) {
  var children = _ref.children,
      className = _ref.className,
      muted = _ref.muted,
      gray = _ref.gray,
      graydark = _ref.graydark,
      nounderline = _ref.nounderline,
      rest = objectWithoutProperties(_ref, ['children', 'className', 'muted', 'gray', 'graydark', 'nounderline']);
  return React.createElement(
    'a',
    _extends({
      className: classnames(className, 'text-blue', {
        'muted-link': muted,
        'link-gray': gray,
        'link-gray-dark': graydark,
        'no-underline': nounderline
      })
    }, rest),
    children
  );
};

var stateColorMap = {
  open: 'green',
  opened: 'green',
  reopened: 'green',
  closed: 'red',
  merged: 'purple'
};

var stateOcticonMap = {
  open: IssueOpened,
  opened: IssueOpened,
  reopened: IssueReopened,
  closed: IssueClosed,
  merged: GitMerge
};

function getOcticon(state) {
  if (!state) {
    return null;
  }
  return React.createElement(Octicon, { icon: stateOcticonMap[state] });
}

function getIconComponent(icon, children) {
  if (icon && children) {
    return React.createElement(
      'span',
      { className: 'mr-1' },
      icon
    );
  } else if (icon) {
    return React.createElement(
      'span',
      { className: 'd-flex m-1' },
      icon
    );
  }
  return null;
}

var StateLabel = function StateLabel(_ref) {
  var state = _ref.state,
      scheme = _ref.scheme,
      small = _ref.small,
      icon = _ref.icon,
      children = _ref.children;

  if (icon !== false) {
    icon = icon || getOcticon(state);
  }

  var color = scheme || stateColorMap[state];
  var styleProps = {};
  if (color === 'yellow') {
    styleProps.style = { backgroundColor: colors.yellow[7] };
  }
  var iconComponent = getIconComponent(icon, children);
  return React.createElement(
    'span',
    _extends({
      className: classnames('State', {
        'State--small': small
      }, color && color !== 'yellow' ? 'State--' + color : null)
    }, styleProps),
    iconComponent,
    children
  );
};

StateLabel.propTypes = {
  children: PropTypes.node,
  icon: PropTypes.oneOfType([PropTypes.node, PropTypes.bool]),
  scheme: PropTypes.string,
  small: PropTypes.bool,
  state: PropTypes.oneOf(Object.keys(stateOcticonMap))
};

var stateColorMap$1 = {
  ready: 'green',
  invalid: 'invalid',
  merged: 'purple',
  pending: 'yellow'
};

var octicon = React.createElement(Octicon, { icon: GitMerge, size: 'medium' });

var MergeStatus = function MergeStatus(_ref) {
  var state = _ref.state;
  return React.createElement(StateLabel, { scheme: stateColorMap$1[state], icon: octicon });
};

MergeStatus.propTypes = {
  state: PropTypes.oneOf(['ready', 'invalid', 'merged', 'pending']).isRequired
};

var alignmentDirections = ['ne', 'se', 'nw', 'sw'];

var Tooltip = function Tooltip(_ref) {
  var children = _ref.children,
      direction = _ref.direction,
      text = _ref.text,
      noDelay = _ref.noDelay,
      align = _ref.align,
      wrap = _ref.wrap;
  return React.createElement(
    'span',
    {
      'aria-label': text,
      className: classnames('tooltipped', 'tooltipped-' + direction, align && 'tooltipped-align-' + align + '-2', noDelay && 'tooltipped-no-delay', wrap && 'tooltipped-multiline')
    },
    children
  );
};

Tooltip.directions = ['n', 'ne', 'e', 'se', 's', 'sw', 'w', 'nw'];

Tooltip.defaultProps = {
  direction: 'n'
};

Tooltip.propTypes = {
  align: PropTypes.oneOf(alignmentDirections),
  children: PropTypes.node,
  direction: PropTypes.oneOf(Tooltip.directions),
  noDelay: PropTypes.bool,
  text: PropTypes.string,
  wrap: PropTypes.bool
};

var CounterLabel = function CounterLabel(_ref) {
  var theme = _ref.theme,
      children = _ref.children;
  return React.createElement(
    'span',
    { className: classnames('Counter', theme ? 'Counter--' + theme : null) },
    children
  );
};

CounterLabel.propTypes = {
  children: PropTypes.node,
  theme: PropTypes.oneOf(['gray', 'gray-light'])
};

var Flash = function Flash(_ref) {
  var children = _ref.children,
      className = _ref.className,
      full = _ref.full,
      yellow = _ref.yellow,
      red = _ref.red,
      green = _ref.green;
  return React.createElement(
    'div',
    {
      className: classnames(className, 'flash', {
        'flash-full': full,
        'flash-warn': yellow,
        'flash-error': red,
        'flash-success': green
      })
    },
    children
  );
};

export { theme, Block, Box, Avatar, Button, ButtonDanger, ButtonPrimary, ButtonOutline, ButtonLink, Caret, CaretBox, CircleOcticon, Details, Dropdown, DonutChart, DonutSlice, FlexContainer, FlexItem, TextInput, Heading, Label, BranchName, Link, MergeStatus, Text, Tooltip, CounterLabel, Flash, StateLabel };
